@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 Managing Gateway Groups and interactions with multiple channels.
 
@@ -9,23 +8,23 @@ import sys
 from functools import partial
 from threading import Lock
 
-from execnet import gateway_bootstrap
-from execnet import gateway_io
-from execnet import XSpec
-from execnet.gateway_base import get_execmodel
-from execnet.gateway_base import reraise
-from execnet.gateway_base import trace
+from . import gateway_bootstrap
+from . import gateway_io
+from .gateway_base import get_execmodel
+from .gateway_base import trace
+from .gateway_base import WorkerPool
+from .xspec import XSpec
 
 NO_ENDMARKER_WANTED = object()
 
 
-class Group(object):
-    """ Gateway Groups. """
+class Group:
+    """Gateway Groups."""
 
     defaultspec = "popen"
 
     def __init__(self, xspecs=(), execmodel="thread"):
-        """ initialize group and make gateways as specified.
+        """initialize group and make gateways as specified.
         execmodel can be 'thread' or 'eventlet'.
         """
         self._gateways = []
@@ -51,7 +50,7 @@ class Group(object):
         return self._remote_execmodel
 
     def set_execmodel(self, execmodel, remote_execmodel=None):
-        """ Set the execution model for local and remote site.
+        """Set the execution model for local and remote site.
 
         execmodel can be one of "thread" or "eventlet" (XXX gevent).
         It determines the execution model for any newly created gateway.
@@ -133,12 +132,12 @@ class Group(object):
             io = gateway_io.create_io(spec, execmodel=self.execmodel)
             gw = gateway_bootstrap.bootstrap(io, spec)
         elif spec.socket:
-            from execnet import gateway_socket
+            from . import gateway_socket
 
             io = gateway_socket.create_io(spec, self, execmodel=self.execmodel)
             gw = gateway_bootstrap.bootstrap(io, spec)
         else:
-            raise ValueError("no gateway type found for {!r}".format(spec._spec))
+            raise ValueError(f"no gateway type found for {spec._spec!r}")
         gw.spec = spec
         self._register(gw)
         if spec.chdir or spec.nice or spec.env:
@@ -163,19 +162,19 @@ class Group(object):
         return gw
 
     def allocate_id(self, spec):
-        """ (re-entrant) allocate id for the given xspec object. """
+        """(re-entrant) allocate id for the given xspec object."""
         if spec.id is None:
             with self._autoidlock:
                 id = "gw" + str(self._autoidcounter)
                 self._autoidcounter += 1
                 if id in self:
-                    raise ValueError("already have gateway with id {!r}".format(id))
+                    raise ValueError(f"already have gateway with id {id!r}")
                 spec.id = id
 
     def _register(self, gateway):
         assert not hasattr(gateway, "_group")
         assert gateway.id
-        assert id not in self
+        assert gateway.id not in self
         self._gateways.append(gateway)
         gateway._group = self
 
@@ -184,11 +183,11 @@ class Group(object):
         self._gateways_to_join.append(gateway)
 
     def _cleanup_atexit(self):
-        trace("=== atexit cleanup {!r} ===".format(self))
+        trace(f"=== atexit cleanup {self!r} ===")
         self.terminate(timeout=1.0)
 
     def terminate(self, timeout=None):
-        """ trigger exit of member gateways and wait for termination
+        """trigger exit of member gateways and wait for termination
         of member gateways and associated subprocesses.  After waiting
         timeout seconds try to to kill local sub processes of popen-
         and ssh-gateways.  Timeout defaults to None meaning
@@ -223,8 +222,8 @@ class Group(object):
             self._gateways_to_join[:] = []
 
     def remote_exec(self, source, **kwargs):
-        """ remote_exec source on all member gateways and return
-            MultiChannel connecting to all sub processes.
+        """remote_exec source on all member gateways and return
+        MultiChannel connecting to all sub processes.
         """
         channels = []
         for gw in self:
@@ -290,17 +289,17 @@ class MultiChannel:
                 if first is None:
                     first = sys.exc_info()
         if first:
-            reraise(*first)
+            raise first[1].with_traceback(first[2])
 
 
 def safe_terminate(execmodel, timeout, list_of_paired_functions):
-    workerpool = execmodel.WorkerPool()
+    workerpool = WorkerPool(execmodel)
 
     def termkill(termfunc, killfunc):
         termreply = workerpool.spawn(termfunc)
         try:
             termreply.get(timeout=timeout)
-        except IOError:
+        except OSError:
             killfunc()
 
     replylist = []
