@@ -1,16 +1,15 @@
-# -*- coding: utf-8 -*-
 """
 (c) 2008-2013, holger krekel
 """
 
 
 class XSpec:
-    """ Execution Specification: key1=value1//key2=value2 ...
-        * keys need to be unique within the specification scope
-        * neither key nor value are allowed to contain "//"
-        * keys are not allowed to contain "="
-        * keys are not allowed to start with underscore
-        * if no "=value" is given, assume a boolean True value
+    """Execution Specification: key1=value1//key2=value2 ...
+    * keys need to be unique within the specification scope
+    * neither key nor value are allowed to contain "//"
+    * keys are not allowed to contain "="
+    * keys are not allowed to start with underscore
+    * if no "=value" is given, assume a boolean True value
     """
 
     # XXX allow customization, for only allow specific key names
@@ -30,7 +29,7 @@ class XSpec:
             if key[0] == "_":
                 raise AttributeError("%r not a valid XSpec key" % key)
             if key in self.__dict__:
-                raise ValueError("duplicate key: {!r} in {!r}".format(key, string))
+                raise ValueError(f"duplicate key: {key!r} in {string!r}")
             if key.startswith("env:"):
                 self.env[key[4:]] = value
             else:
@@ -42,7 +41,7 @@ class XSpec:
         return None
 
     def __repr__(self):
-        return "<XSpec {!r}>".format(self._spec)
+        return f"<XSpec {self._spec!r}>"
 
     def __str__(self):
         return self._spec
